import os
from aws_cdk import (
    core,
)

from acru_l.resources.custom_resources import PythonCustomResource

canary_dirname = os.path.dirname(__file__)


class Canary(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        deploy_id: str,
        version: str,
        health_check_url: str
    ):
        super().__init__(scope, id)
        self.custom_resource = PythonCustomResource(
            self,
            "CustomResource",
            source_dir=os.path.join(canary_dirname, "src"),
            index="handler.py",
            handler="main",
            resource_properties={
                "deploy_id": deploy_id,
                "version": version,
                "health_check_url": health_check_url,
            },
        )

    def add_dependency(self, resource: core.Resource):
        self.custom_resource.resource.node.add_dependency(resource)

    @property
    def resource(self):
        return self.custom_resource.resource



from typing import Type, Optional, Mapping, Dict, List, Any

import pydantic
import toml
from aws_cdk import core

from acru_l.utils import traverse


class Settings(pydantic.BaseSettings):

    AWS_ACCOUNT_ID: str
    AWS_REGION: str
    DEPLOY_ID: str
    ACRUL_CONFIG_PATH: pydantic.FilePath = "./acru-l.toml"
    ACRUL_SECTION: Optional[str] = None

    class Config:
        case_sensitive = True
        env_file = ".env"
        env_file_encoding = "utf-8"

    @property
    def env(self):
        return core.Environment(
            account=self.AWS_ACCOUNT_ID, region=self.AWS_REGION
        )

    @property
    def config(self):
        data = toml.load(self.ACRUL_CONFIG_PATH)
        if self.ACRUL_SECTION:
            data = traverse(data, self.ACRUL_SECTION)
        return AcrulConfig(**data)


class StackConfig(pydantic.BaseModel):

    id: str
    factory: pydantic.PyObject
    options: Optional[Dict]

    analytics_reporting: Optional[bool] = None
    description: Optional[str] = None
    name: Optional[str] = None
    synthesizer: Optional[pydantic.PyObject] = None
    tags: Optional[Mapping[str, str]] = None
    termination_protection: Optional[bool] = None


class AppConfig(pydantic.BaseModel):
    analytics_reporting: Optional[bool] = None
    auto_synth: Optional[bool] = None
    context: Optional[Mapping[str, Any]] = None
    outdir: Optional[str] = None
    runtime_info: Optional[bool] = None
    stack_traces: Optional[bool] = None
    tree_metadata: Optional[bool] = None


class AcrulConfig(pydantic.BaseModel):
    app: AppConfig = AppConfig()
    stacks: List[StackConfig]


def app_factory(
    account: Optional[str] = None,
    region: Optional[str] = None,
    deploy_id: Optional[str] = None,
    config_path: Optional[str] = None,
    section: Optional[str] = None,
) -> "App":
    default_settings = {}
    if account:
        default_settings["AWS_ACCOUNT_ID"] = account
    if region:
        default_settings["AWS_REGION"] = region
    if deploy_id:
        default_settings["DEPLOY_ID"] = deploy_id
    if config_path:
        default_settings["ACRUL_CONFIG_PATH"] = config_path
    if section:
        default_settings["ACRUL_SECTION"] = section

    settings = Settings(**default_settings)
    env = settings.env
    config = settings.config
    app = App(
        analytics_reporting=config.app.analytics_reporting,
        auto_synth=config.app.auto_synth,
        context=config.app.context,
        outdir=config.app.outdir,
        runtime_info=config.app.runtime_info,
        stack_traces=config.app.stack_traces,
        tree_metadata=config.app.tree_metadata,
    )

    for stack in config.stacks:
        app.add_stack(
            stack.factory(),
            stack.id,
            deploy_id=settings.DEPLOY_ID,
            env=env,
            description=stack.description,
            analytics_reporting=stack.analytics_reporting,
            stack_name=stack.name,
            synthesizer=stack.synthesizer,
            tags=stack.tags,
            termination_protection=stack.termination_protection,
            options=stack.options,
        )

    return app


class App(core.App):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._stacks = {}

    def add_stack(
        self,
        stack_factory: "StackFactory",
        id: str,
        *,
        deploy_id: str,
        analytics_reporting: Optional[bool] = None,
        description: Optional[str] = None,
        env: Optional[core.Environment] = None,
        stack_name: Optional[str] = None,
        synthesizer: Optional[core.IStackSynthesizer] = None,
        tags: Optional[Mapping[str, str]] = None,
        termination_protection: Optional[bool] = None,
        options: Optional[Dict] = None
    ) -> "Stack":
        self._stacks[id] = stack_factory.build(
            self,
            id,
            deploy_id=deploy_id,
            analytics_reporting=analytics_reporting,
            description=description,
            env=env,
            stack_name=stack_name,
            synthesizer=synthesizer,
            tags=tags,
            termination_protection=termination_protection,
            options=options,
        )
        return self._stacks[id]


class Stack(core.Stack):
    """
    Usage:
        class MyStack(Stack):

            def build(self, options: MyOptionsClass):
                ...


        app = App()
        add.add_stack(
            MyStack,
            "MyStackId",

    """

    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        deploy_id: str,
        analytics_reporting: Optional[bool] = None,
        description: Optional[str] = None,
        env: Optional[core.Environment] = None,
        stack_name: Optional[str] = None,
        synthesizer: Optional[core.IStackSynthesizer] = None,
        tags: Optional[Mapping[str, str]] = None,
        termination_protection: Optional[bool] = None,
        options: Optional[pydantic.BaseModel] = None
    ):
        super().__init__(
            scope,
            id,
            env=env,
            analytics_reporting=analytics_reporting,
            description=description,
            stack_name=stack_name,
            synthesizer=synthesizer,
            tags=tags,
            termination_protection=termination_protection,
        )
        self.deploy_id = deploy_id
        self.build(options=options)

    def build(self, options: pydantic.BaseModel):
        pass  # pragma: no cover


class StackFactory:
    stack_class: Type[Stack]
    options_class: Type[pydantic.BaseModel]

    def build(
        self,
        scope: core.Construct,
        id: str,
        *,
        deploy_id: str,
        analytics_reporting: Optional[bool] = None,
        description: Optional[str] = None,
        env: Optional[core.Environment] = None,
        stack_name: Optional[str] = None,
        synthesizer: Optional[core.IStackSynthesizer] = None,
        tags: Optional[Mapping[str, str]] = None,
        termination_protection: Optional[bool] = None,
        options: Optional[Dict] = None
    ) -> Stack:
        return self.stack_class(
            scope,
            id,
            deploy_id=deploy_id,
            env=env,
            analytics_reporting=analytics_reporting,
            description=description,
            stack_name=stack_name,
            synthesizer=synthesizer,
            tags=tags,
            termination_protection=termination_protection,
            options=self.options_class(**options) if options else None,
        )



import os

from typing import Optional, List

from aws_cdk import (
    core,
    custom_resources as acr,
    aws_iam as iam,
    aws_lambda as _lambda,
    aws_lambda_python as lambda_python,
    aws_route53 as route53,
)

from acru_l.resources.custom_resources import PythonCustomResource


dirname = os.path.dirname(__file__)


class SESVerification(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        hosted_zone: route53.HostedZone,
        emails: Optional[List[str]]
    ):
        super().__init__(scope, id)

        ses_config_layer = lambda_python.PythonLayerVersion(
            scope,
            "SESConfigLayer",
            entry=os.path.join(dirname, "layer"),
            compatible_runtimes=[_lambda.Runtime.PYTHON_3_8],
        )

        PythonCustomResource(
            self,
            "SESDomainValidation",
            source_dir=os.path.join(dirname, "domain_validation"),
            handler="on_event",
            environment={
                "DOMAIN": hosted_zone.zone_name,
                "HOSTED_ZONE_ID": hosted_zone.hosted_zone_id,
            },
            layers=[
                ses_config_layer,
            ],
            policy_statements=[
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=[
                        "ses:VerifyDomainDkim",
                        "ses:VerifyDomainIdentity",
                    ],
                    resources=acr.AwsCustomResourcePolicy.ANY_RESOURCE,
                ),
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=["route53:ChangeResourceRecordSets"],
                    resources=[hosted_zone.hosted_zone_arn],
                ),
            ],
        )

        emails = emails or []
        PythonCustomResource(
            self,
            "SESVerifyEmails",
            source_dir=os.path.join(dirname, "verified_emails"),
            handler="on_event",
            layers=[
                ses_config_layer,
            ],
            policy_statements=[
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=["ses:VerifyEmailIdentity"],
                    resources=acr.AwsCustomResourcePolicy.ANY_RESOURCE,
                )
            ],
            resource_properties={"emails": emails},
        )



from typing import List, Optional

from aws_cdk import (
    core,
    aws_ec2 as ec2,
    aws_iam as iam,
    aws_lambda as _lambda,
    aws_lambda_python as lambda_python,
    aws_logs as logs,
    aws_s3 as s3,
)


class FunctionWrapper(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        private_bucket: Optional[s3.Bucket] = None,
        secret_arns: Optional[List] = None,
        policy_statements: Optional[List[iam.PolicyStatement]] = None,
    ):
        super().__init__(scope, id)
        self.secret_arns = secret_arns
        self.policy_statements = policy_statements or []
        self.private_bucket = private_bucket

    def setup_function_perms(self):
        if self.secret_arns:
            self.handler.add_to_role_policy(
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=["secretsmanager:GetSecretValue"],
                    resources=self.secret_arns,
                )
            )
        if self.private_bucket:
            self.handler.add_to_role_policy(
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=[
                        "s3:PutObject",
                        "s3:GetObjectAcl",
                        "s3:GetObject",
                        "s3:ListBucket",
                        "s3:DeleteObject",
                        "s3:PutObjectAcl",
                    ],
                    resources=[
                        f"{self.private_bucket.bucket_arn}/*",
                        self.private_bucket.bucket_arn,
                    ],
                )
            )

        for policy_statement in self.policy_statements:
            self.handler.add_to_role_policy(policy_statement)


class Function(FunctionWrapper):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        source_path: str,
        handler_path: str = "handler.main",
        layers: List[_lambda.LayerVersion],
        runtime: _lambda.Runtime = _lambda.Runtime.PYTHON_3_8,
        vpc: Optional[ec2.Vpc] = None,
        environment_variables: Optional[dict] = None,
        timeout: core.Duration = core.Duration.seconds(300),
        memory_size: int = 1024,
        log_retention: logs.RetentionDays = logs.RetentionDays.ONE_DAY,
        profiling: bool = False,
        tracing: Optional[_lambda.Tracing] = None,
        **kwargs,
    ):
        super().__init__(scope, id, **kwargs)
        self.handler = _lambda.Function(
            self,
            "Handler",
            code=_lambda.Code.from_asset(source_path),
            handler=handler_path,
            runtime=runtime,
            layers=layers,
            memory_size=memory_size,
            environment=environment_variables,
            timeout=timeout,
            vpc=vpc,
            log_retention=log_retention,
            tracing=tracing,
            profiling=profiling,
        )
        self.setup_function_perms()


class PythonFunction(FunctionWrapper):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        source_path: str,
        index: str = "handler.py",
        handler: str = "main",
        layers: List[_lambda.LayerVersion],
        runtime: _lambda.Runtime = _lambda.Runtime.PYTHON_3_8,
        vpc: Optional[ec2.Vpc] = None,
        environment_variables: Optional[dict] = None,
        timeout: core.Duration = core.Duration.seconds(300),
        memory_size: int = 1024,
        log_retention: logs.RetentionDays = logs.RetentionDays.ONE_DAY,
        profiling: bool = False,
        tracing: Optional[_lambda.Tracing] = None,
        **kwargs,
    ):
        super().__init__(scope, id, **kwargs)
        self.handler = lambda_python.PythonFunction(
            self,
            "Handler",
            entry=source_path,
            handler=handler,
            index=index,
            runtime=runtime,
            layers=layers,
            memory_size=memory_size,
            environment=environment_variables,
            timeout=timeout,
            vpc=vpc,
            log_retention=log_retention,
            tracing=tracing,
            profiling=profiling,
        )
        self.setup_function_perms()



from typing import Optional

from pydantic import BaseModel

from acru_l.core import Stack, StackFactory
from acru_l.resources.vpc import VPC
from acru_l.stacks.certs import HostedZoneFactory


class VpcOptions(BaseModel):
    name: str
    cidr: str
    export_name: str


class NetworkOptions(BaseModel):
    vpc: VpcOptions
    hosted_zone: Optional[HostedZoneFactory] = None


class NetworkStack(Stack):
    def build(self, options: NetworkOptions):
        VPC(
            self,
            "VPC",
            name=options.vpc.name,
            cidr=options.vpc.cidr,
            export_name=options.vpc.export_name,
        )
        if options.hosted_zone is not None:
            options.hosted_zone.build(self)


class NetworkStackFactory(StackFactory):
    stack_class = NetworkStack
    options_class = NetworkOptions



import os
import subprocess
import sys

from dotenv import load_dotenv


load_dotenv()


def main():
    dirname = os.path.dirname(__file__)
    process = subprocess.run(
        ["cdk", f"--app={os.path.join(dirname, 'app.py')}"] + sys.argv[1:]
    )
    sys.exit(process.returncode)



import boto3
import time

from acrul_toolkit.custom_resources import CustomResourceEventHandler

ses = boto3.client("ses")


def verify_emails(email_addresses=None):
    email_addresses = email_addresses or []
    for email_address in email_addresses:
        ses.verify_email_identity(EmailAddress=email_address)
        time.sleep(1)


class EventHandler(CustomResourceEventHandler):
    @staticmethod
    def get_emails(event):
        return event["ResourceProperties"]["emails"]

    def on_create(self, event):
        verify_emails(self.get_emails(event))

    def on_update(self, event):
        verify_emails(self.get_emails(event))

    def on_delete(self, event):
        pass


on_event = EventHandler()



from typing import Optional, List

from aws_cdk import core, aws_dynamodb as ddb


class DynamoDB(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        table_name: str,
        billing_mode: ddb.BillingMode = ddb.BillingMode.PAY_PER_REQUEST,
        stream: ddb.StreamViewType = ddb.StreamViewType.NEW_AND_OLD_IMAGES,
        replication_regions: Optional[List[str]] = None,
        **kwargs
    ):
        super().__init__(scope, id)
        self.table = ddb.Table(
            self,
            "Table",
            table_name=table_name,
            partition_key=ddb.Attribute(
                name="partition_key", type=ddb.AttributeType.STRING
            ),
            sort_key=ddb.Attribute(
                name="sort_key", type=ddb.AttributeType.STRING
            ),
            billing_mode=billing_mode,
            stream=stream,
            replication_regions=replication_regions,
            **kwargs
        )



from typing import List, Optional

from aws_cdk import aws_route53 as route53
from pydantic import BaseModel

from acru_l.core import Stack, StackFactory
from acru_l.resources.ses import SESVerification


class SESOptions(BaseModel):
    hosted_zone_domain_name: str
    emails: Optional[List[str]] = None


class EmailsStack(Stack):
    def build(self, options: SESOptions):
        hosted_zone = route53.HostedZone.from_lookup(
            self, "HostedZone", domain_name=options.hosted_zone_domain_name
        )
        SESVerification(
            self,
            "Verification",
            hosted_zone=hosted_zone,
            emails=options.emails,
        )


class EmailsStackFactory(StackFactory):
    stack_class = EmailsStack
    options_class = SESOptions



from aws_cdk import (
    core,
    aws_apigatewayv2 as apigateway,
    aws_apigatewayv2_integrations as apigateway_integrations,
    aws_lambda as _lambda,
    aws_certificatemanager as acm,
    aws_route53 as rout53,
)


class LambdaAPIGateway(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        domain_name: str,
        certificate: acm.Certificate,
        handler: _lambda.Function,
        hosted_zone: rout53.HostedZone,
        payload_format_version: apigateway.PayloadFormatVersion = apigateway.PayloadFormatVersion.VERSION_1_0,  # noqa: E501
    ):
        super().__init__(scope, id)
        integration = apigateway_integrations.LambdaProxyIntegration(
            handler=handler, payload_format_version=payload_format_version
        )
        dn = apigateway.DomainName(
            self,
            "DomainName",
            domain_name=domain_name,
            certificate=certificate,
        )
        self.api = apigateway.HttpApi(
            self,
            "Gateway",
            default_integration=integration,
            cors_preflight=apigateway.CorsPreflightOptions(
                allow_headers=[
                    "Accept",
                    "Accept-Encoding",
                    "Authorization",
                    "Content-Type",
                    "Dnt",
                    "Origin",
                    "User-Agent",
                    "X-CSRFToken",
                    "X-Requested-With",
                ],
                allow_methods=[
                    apigateway.HttpMethod.POST,
                    apigateway.HttpMethod.GET,
                    apigateway.HttpMethod.PATCH,
                    apigateway.HttpMethod.PUT,
                    apigateway.HttpMethod.DELETE,
                    apigateway.HttpMethod.HEAD,
                    apigateway.HttpMethod.OPTIONS,
                ],
                allow_credentials=False,
                allow_origins=["*"],
                max_age=core.Duration.minutes(60),
            ),
            default_domain_mapping=apigateway.DefaultDomainMappingOptions(
                domain_name=dn,
            ),
        )

        rout53.CnameRecord(
            self,
            "Record",
            domain_name=dn.regional_domain_name,
            zone=hosted_zone,
            record_name=f"{domain_name}.",
        )



from acrul_toolkit.custom_resources import CustomResourceEventHandler
import requests


class EventHandler(CustomResourceEventHandler):
    def on_create(self, event):
        self.check_api_endpoint(event)

    def on_update(self, event):
        self.check_api_endpoint(event)

    def on_delete(self, event):
        pass

    @staticmethod
    def check_api_endpoint(event):
        url = event["ResourceProperties"]["health_check_url"]
        tries = 0
        error = None
        while tries < 3:
            response = requests.get(url)
            if response.status_code == 200:
                error = None
                break
            error = f"{response.status_code}: {response.text}"
            tries += 1

        if error is not None:
            raise RuntimeError(f"Check Failed: {error}")


main = EventHandler()



import enum
import os
from typing import cast, List, Optional, Union, Mapping, Dict

from aws_cdk import (
    core,
    aws_certificatemanager as acm,
    aws_cognito as cognito,
    aws_iam as iam,
    aws_lambda as _lambda,
    aws_logs as logs,
)
from aws_cdk.aws_lambda_python import PythonFunction, PythonLayerVersion
from pydantic import (
    BaseModel,
    PyObject,
    EmailStr,
    Field,
    FilePath,
    DirectoryPath,
)


class Factory(BaseModel):
    def build(self, *args, **kwargs):
        pass


def _build(conf: Optional[Factory], *args, **kwargs):
    if not conf:
        return
    return conf.build(*args, **kwargs)


class UserPool(core.Construct):
    def __init__(
        self, scope: core.Construct, id: str, *, options: "UserPoolOptions"
    ):
        super().__init__(scope, id)

        self.pool = cognito.UserPool(
            self,
            "UserPool",
            user_pool_name=options.user_pool_name,
            account_recovery=options.account_recovery,
            auto_verify=_build(options.auto_verify),
            custom_attributes={
                attr.name: attr.build() for attr in options.custom_attributes
            },
            mfa=options.mfa,
            mfa_second_factor=_build(options.mfa_second_factor),
            password_policy=_build(options.password_policy),
            self_sign_up_enabled=options.self_signup_enabled,
            sign_in_aliases=_build(options.sign_in_aliases),
            sign_in_case_sensitive=options.sign_in_case_sensitive,
            standard_attributes=_build(options.standard_attributes),
            user_invitation=_build(options.invitation_options),
            user_verification=_build(options.verification_options),
        )

        _build(options.email_configuration, self.pool)

        self.triggers = {}

        for trigger in options.python_triggers:
            operation, fn = trigger.build(scope=self, pool=self.pool)
            self.triggers[operation] = fn

        for app_client in options.app_clients:
            app_client.build(self, self.pool)

        if options.app_clients:
            _build(options.domain_options, self, self.pool)


class AppClientFactory(Factory):

    name: str
    export_name: str
    admin_user_password: Optional[bool] = None
    user_password: Optional[bool] = None
    user_srp: Optional[bool] = None
    prevent_user_existence_errors: Optional[bool] = None

    def build(self, scope: core.Construct, pool: cognito.UserPool):
        client = pool.add_client(
            self.name,
            auth_flows=cognito.AuthFlow(
                admin_user_password=self.admin_user_password,
                user_srp=self.user_srp,
                user_password=self.user_password,
            ),
            prevent_user_existence_errors=self.prevent_user_existence_errors,
        )
        core.CfnOutput(
            scope,
            self.export_name,
            value=client.user_pool_client_id,
            export_name=self.export_name,
        )


class Operation(enum.Enum):
    CREATE_AUTH_CHALLENGE: str = "CREATE_AUTH_CHALLENGE"
    CUSTOM_MESSAGE: str = "CUSTOM_MESSAGE"
    DEFINE_AUTH_CHALLENGE: str = "DEFINE_AUTH_CHALLENGE"
    POST_AUTHENTICATION: str = "POST_AUTHENTICATION"
    POST_CONFIRMATION: str = "POST_CONFIRMATION"
    PRE_AUTHENTICATION: str = "PRE_AUTHENTICATION"
    PRE_SIGN_UP: str = "PRE_SIGN_UP"
    PRE_TOKEN_GENERATION: str = "PRE_TOKEN_GENERATION"
    USER_MIGRATION: str = "USER_MIGRATION"
    VERIFY_AUTH_CHALLENGE_RESPONSE: str = "VERIFY_AUTH_CHALLENGE_RESPONSE"

    @property
    def instance(self):
        return getattr(cognito.UserPoolOperation, self.value)


class PythonFunctionLayerFactory(Factory):
    name: str
    arn_export_name: Optional[str] = None
    source_path: Optional[DirectoryPath] = None

    def build(self, scope: core.Construct, runtime: _lambda.Runtime):
        if self.arn_export_name:
            arn = core.Fn.import_value(self.arn_export_name)
            return PythonLayerVersion.from_layer_version_arn(
                scope, self.name, layer_version_arn=arn
            )
        return PythonLayerVersion(
            scope,
            self.name,
            entry=str(self.source_path),
            compatible_runtimes=[runtime],
        )


class PythonFunctionFactory(Factory):
    name: str
    source_path: DirectoryPath
    export_name: str

    runtime: Optional[PyObject] = None
    index: str = "handler.py"
    handler: str = "main"
    memory_size: int = 512
    profiling: bool = True
    tracing: Optional[_lambda.Tracing] = None
    layers: List[PythonFunctionLayerFactory] = Field(default_factory=list)
    description: Optional[str] = None
    environment: Optional[Dict[str, str]] = Field(default_factory=dict)
    local_environment_names: List[str] = Field(default_factory=list)
    log_retention: logs.RetentionDays = logs.RetentionDays.ONE_DAY
    pool_actions: List[str] = Field(default_factory=list)
    retry_attempts: int = 2

    def build(self, scope: core.Construct, pool: cognito.UserPool):
        runtime = self.runtime or _lambda.Runtime.PYTHON_3_8
        layers = [opts.build(scope, runtime) for opts in self.layers]
        for name in self.local_environment_names:
            self.environment[name] = os.environ[name]
        fn = PythonFunction(
            scope,
            self.name,
            runtime=runtime,
            entry=str(self.source_path),
            index=self.index,
            handler=self.handler,
            layers=layers,
            description=self.description,
            environment=self.environment,
            log_retention=self.log_retention,
            profiling=self.profiling,
            memory_size=self.memory_size,
            tracing=self.tracing,
        )
        if self.pool_actions:
            policy = iam.Policy(
                scope,
                f"{self.name}PoolPolicy",
                statements=[
                    iam.PolicyStatement(
                        actions=self.pool_actions,
                        resources=[pool.user_pool_arn],
                    )
                ],
            )
            fn.role.attach_inline_policy(policy)

        core.CfnOutput(
            scope,
            f"{self.name}Output",
            value=fn.function_arn,
            export_name=self.export_name,
            description=self.description,
        )
        return fn


class PythonTriggerFactory(Factory):
    operation: Operation
    function_options: PythonFunctionFactory

    def build(self, scope: core.Construct, pool: cognito.UserPool):
        fn = self.function_options.build(scope, pool)
        pool.add_trigger(operation=self.operation.instance, fn=fn)
        return self.operation, fn


class SignInAliases(Factory):
    email: Optional[bool] = None
    phone: Optional[bool] = None
    preferred_username: Optional[bool] = None
    username: Optional[bool] = None

    def build(self):
        return cognito.SignInAliases(
            email=self.email,
            phone=self.phone,
            preferred_username=self.preferred_username,
            username=self.username,
        )


class CustomAttribute(Factory):
    name: str
    attr_class: PyObject
    mutable: bool = False
    extra: Optional[Mapping[str, Union[float, int]]] = Field(
        default_factory=dict
    )

    def build(self):
        return self.attr_class(mutable=self.mutable, **self.extra)


class AutoVerify(Factory):
    email: bool = True
    phone: bool = True

    def build(self):
        return cognito.AutoVerifiedAttrs(email=self.email, phone=self.phone)


class PasswordPolicy(Factory):
    min_length: Optional[int] = None
    require_digits: Optional[bool] = None
    require_lowercase: Optional[bool] = None
    require_symbols: Optional[bool] = None
    require_uppercase: Optional[bool] = None
    temp_password_validity: Optional[int] = None

    def build(self):
        return cognito.PasswordPolicy(
            min_length=self.min_length,
            require_digits=self.require_digits,
            require_lowercase=self.require_lowercase,
            require_symbols=self.require_symbols,
            require_uppercase=self.require_uppercase,
            temp_password_validity=core.Duration.days(
                self.temp_password_validity
            )
            if self.temp_password_validity
            else None,
        )


class StandardAttribute(Factory):
    mutable: Optional[bool] = None
    required: Optional[bool] = None

    def build(self):
        return cognito.StandardAttribute(
            mutable=self.mutable, required=self.required
        )


class StandardAttributes(Factory):
    address: Optional[StandardAttribute] = None
    birthdate: Optional[StandardAttribute] = None
    email: Optional[StandardAttribute] = None
    family_name: Optional[StandardAttribute] = None
    fullname: Optional[StandardAttribute] = None
    gender: Optional[StandardAttribute] = None
    given_name: Optional[StandardAttribute] = None
    last_update_time: Optional[StandardAttribute] = None
    locale: Optional[StandardAttribute] = None
    middle_name: Optional[StandardAttribute] = None
    nickname: Optional[StandardAttribute] = None
    phone_number: Optional[StandardAttribute] = None
    preferred_username: Optional[StandardAttribute] = None
    profile_page: Optional[StandardAttribute] = None
    profile_picture: Optional[StandardAttribute] = None
    timezone: Optional[StandardAttribute] = None
    website: Optional[StandardAttribute] = None

    def build(self):
        return cognito.StandardAttributes(
            address=_build(self.address),
            birthdate=_build(self.birthdate),
            email=_build(self.email),
            family_name=_build(self.family_name),
            fullname=_build(self.fullname),
            gender=_build(self.gender),
            given_name=_build(self.given_name),
            last_update_time=_build(self.last_update_time),
            locale=_build(self.locale),
            middle_name=_build(self.middle_name),
            nickname=_build(self.nickname),
            phone_number=_build(self.phone_number),
            preferred_username=_build(self.preferred_username),
            profile_page=_build(self.profile_page),
            profile_picture=_build(self.profile_picture),
            timezone=_build(self.timezone),
            website=_build(self.website),
        )


class EmailSettings(Factory):
    source_arn: Optional[str] = None
    from_: Optional[str] = None
    from_email: Optional[EmailStr] = None
    reply_to_email_address: Optional[str] = None
    email_sending_account: Optional[str] = None
    configuration_set: Optional[str] = None

    def build(self, pool: cognito.UserPool):
        region = pool.stack.region
        account = pool.stack.account
        email = self.from_email
        if email:
            source_arn = f"arn:aws:ses:{region}:{account}:identity/{email}"
            from_ = None
            email_sending_account = "DEVELOPER"
        else:
            source_arn = self.source_arn
            from_ = self.from_
            email_sending_account = self.email_sending_account

        conf = cognito.CfnUserPool.EmailConfigurationProperty(
            source_arn=source_arn,
            from_=from_,
            reply_to_email_address=self.reply_to_email_address,
            email_sending_account=email_sending_account,
            configuration_set=self.configuration_set,
        )
        cfn_pool = cast(cognito.CfnUserPool, pool.node.default_child)
        cfn_pool.email_configuration = conf
        return conf


class InvitationFactory(Factory):

    subject: Optional[FilePath] = None
    body: Optional[FilePath] = None
    sms: Optional[FilePath] = None

    def build(self):
        return cognito.UserInvitationConfig(
            email_subject=self.subject.read_text(),
            email_body=self.body.read_text(),
            sms_message=self.sms.read_text(),
        )


class VerificationFactory(Factory):

    subject: Optional[FilePath] = None
    body: Optional[FilePath] = None
    sms: Optional[FilePath] = None
    email_style: Optional[cognito.VerificationEmailStyle] = None

    def build(self):
        return cognito.UserVerificationConfig(
            email_subject=self.subject.read_text(),
            email_body=self.body.read_text(),
            sms_message=self.sms.read_text(),
            email_style=self.email_style,
        )


class MFASecondFactorFactory(Factory):

    sms: Optional[bool] = None
    otp: Optional[bool] = None

    def build(self):
        return cognito.MfaSecondFactor(otp=self.otp, sms=self.sms)


class DomainFactory(Factory):
    name: str
    domain_name: str
    cert_export_name: str

    def build(self, scope: core.Construct, pool: cognito.UserPool):
        certificate = acm.Certificate.from_certificate_arn(
            scope,
            "Certificate",
            core.Fn.import_value(self.cert_export_name),
        )
        pool.add_domain(
            self.name,
            custom_domain=cognito.CustomDomainOptions(
                domain_name=self.domain_name, certificate=certificate
            ),
        )


class UserPoolOptions(BaseModel):
    user_pool_name: str
    self_signup_enabled: bool = True
    sign_in_case_sensitive: bool = True
    sign_in_aliases: Optional[SignInAliases] = None
    account_recovery: cognito.AccountRecovery = (
        cognito.AccountRecovery.EMAIL_ONLY
    )
    auto_verify: AutoVerify = AutoVerify()
    mfa: Optional[cognito.Mfa] = cognito.Mfa.OFF
    mfa_second_factor: Optional[MFASecondFactorFactory] = None

    password_policy: Optional[PasswordPolicy] = None
    standard_attributes: Optional[StandardAttributes] = None
    custom_attributes: List[CustomAttribute] = Field(default_factory=list)
    email_configuration: Optional[EmailSettings] = None

    invitation_options: Optional[InvitationFactory] = None
    verification_options: Optional[VerificationFactory] = None

    python_triggers: List[PythonTriggerFactory] = Field(default_factory=list)

    app_clients: List[AppClientFactory] = Field(default_factory=list)
    domain_options: Optional[DomainFactory] = None



import os
from typing import List

from aws_cdk import (
    core,
    aws_lambda as _lambda,
    aws_lambda_python as lambda_python,
)

from acru_l.resources.functions import PythonFunction
from acru_l.services.api.base import Service

wsgi_dirname = os.path.dirname(__file__)


class WSGIService(Service):
    def __init__(self, scope: core.Construct, id: str, *, options, **kwargs):
        if not options.api_lambda_source_path:
            options.api_lambda_source_path = os.path.join(wsgi_dirname, "src")
        kwargs["function_class"] = PythonFunction
        super().__init__(scope, id, options=options, **kwargs)

    def package_project(
        self, *, source_path: str
    ) -> List[_lambda.LayerVersion]:
        package_layer = lambda_python.PythonLayerVersion(
            self,
            "ProjectLayer",
            entry=source_path,
            compatible_runtimes=[self.runtime],
        )
        return [package_layer]



import os

from aws_cdk import (
    core,
    aws_sns as sns,
    aws_iam as iam,
)

from acru_l.resources.custom_resources import PythonCustomResource


dirname = os.path.dirname(__file__)


class SESConfigurationSet(core.Construct):
    def __init__(
        self,
        scope: core.Construct,
        id: str,
        *,
        name: str,
        destination_name: str
    ):
        super().__init__(scope, id)
        configuration_set_topic = sns.Topic(self, "Topic")

        PythonCustomResource(
            self,
            "ConfigurationSet",
            source_dir=os.path.join(dirname, "configuration_set"),
            environment={
                "CONFIGURATION_SET_NAME": name,
                "EVENT_DESTINATION_NAME": destination_name,
            },
            resource_properties={
                "TopicARN": configuration_set_topic.topic_arn
            },
            policy_statements=[
                iam.PolicyStatement(
                    effect=iam.Effect.ALLOW,
                    actions=[
                        "ses:CreateConfigurationSet",
                        "ses:DeleteConfigurationSet",
                        "ses:CreateConfigurationSetEventDestination",
                        "ses:UpdateConfigurationSetEventDestination",
                        "ses:DeleteConfigurationSetEventDestination",
                    ],
                    resources=["*"],
                )
            ],
        )



from collections import deque
from typing import Mapping, Any, Union, Deque


def traverse(data: Mapping[str, Any], path: Union[str, Deque]):
    if isinstance(path, str):
        path = path.split(".")
        path = deque(path)
    step = path.popleft()
    data = data[step]
    if path:
        return traverse(data, path)
    return data



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['acru_l',
 'acru_l.resources',
 'acru_l.resources.canary',
 'acru_l.resources.canary.src',
 'acru_l.resources.rds',
 'acru_l.resources.ses',
 'acru_l.resources.ses.configuration_set',
 'acru_l.resources.ses.domain_validation',
 'acru_l.resources.ses.verified_emails',
 'acru_l.services',
 'acru_l.services.api',
 'acru_l.services.api.wsgi',
 'acru_l.services.api.wsgi.src',
 'acru_l.stacks']

package_data = \
{'': ['*'], 'acru_l.resources.ses': ['layer/*']}

install_requires = \
['acru-l-toolkit>=1.0.0-alpha,<2.0.0',
 'aws-cdk.aws-apigatewayv2-integrations==1.79.0',
 'aws-cdk.aws-apigatewayv2==1.79.0',
 'aws-cdk.aws-cloudfront==1.79.0',
 'aws-cdk.aws-dynamodb==1.79.0',
 'aws-cdk.aws-events-targets==1.79.0',
 'aws-cdk.aws-events==1.79.0',
 'aws-cdk.aws-lambda-nodejs==1.79.0',
 'aws-cdk.aws-lambda-python==1.79.0',
 'aws-cdk.aws-lambda==1.79.0',
 'aws-cdk.aws-rds==1.79.0',
 'aws-cdk.aws-route53==1.79.0',
 'aws-cdk.aws-s3==1.79.0',
 'aws-cdk.aws-secretsmanager==1.79.0',
 'aws-cdk.core==1.79.0',
 'aws-cdk.custom-resources==1.79.0',
 'click>=7.1.2,<8.0.0',
 'pydantic[email]>=1.7.3,<2.0.0',
 'python-dotenv>=0.15.0,<0.16.0',
 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['acrul = acru_l.cli:main']}

setup_kwargs = {
    'name': 'acru-l',
    'version': '1.0.0a6',
    'description': '',
    'long_description': '# AWS Cloud Resource Utils - Library (ACRU-L)\n\n[![codecov](https://codecov.io/gh/quadio-media/acru-l/branch/main/graph/badge.svg?token=kWn6eJxFwr)](https://codecov.io/gh/quadio-media/acru-l)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n![Build](https://github.com/quadio-media/acru-l/workflows/Build/badge.svg)\n\nPronounced _Ah-crew-el (*ə-kroo͞′l*)_\n\nAn open source framework for collecting and reusing AWS CDK constructs and stacks.\n\n> NOTE: This project is currently not stable (alpha releases only) and is subject change at any time.\n> Please use at your own risk.\n\n## Usage: ACRU-L Action\n\nThis action provisions AWS stacks given an ACRU-L configuration file. The intention is to encapsulate\nthe code needed to provision resources without conflating application code with devops requirements.\n\nThe goal is to avoid conflating microservice application code with "infrastructure as code".\n\n### Inputs\n\n### `subcommand`\n\n**Optional** The aws-cdk subcommand to run. Default `"deploy -f --require-approval=never"`.\n\n## Example usage\n\n```yaml\n- user: actions/setup-python@v2\n  with:\n  python-version: 3.8\n- uses: actions/setup-node@v2\n  with:\n  node-version: 12\n- uses: quadio-media/acru-l@1.0.0a6\n  with:\n    subcommand: deploy -f\n  env:\n    AWS_REGION: us-east-1\n    AWS_ACCOUNT_ID: ${{ secrets.AWS_ACCOUNT_ID }}\n    AWS_ACCESS_KEY_ID: ${{ secrets.AWS_ACCESS_KEY_ID }}\n    AWS_SECRET_ACCESS_KEY: ${{ secrets.AWS_SECRET_ACCESS_KEY }}\n    DEPLOY_ID: ${{ github.sha }}\n    ACRUL_CONFIG_PATH: "./acru-l.toml"\n```\n\n### Configuration\n\nThe following settings must be passed as environment variables as shown in the example. Sensitive information, especially `AWS_ACCESS_KEY_ID` and `AWS_SECRET_ACCESS_KEY`, should be [set as encrypted secrets](https://help.github.com/en/articles/virtual-environments-for-github-actions#creating-and-using-secrets-encrypted-variables) — otherwise, they\'ll be public to anyone browsing your repository\'s source code and CI logs.\n\n| Key | Value | Suggested Type | Required | Default |\n| ------------- | ------------- | ------------- | ------------- | ------------- |\n| `AWS_ACCOUNT_ID` | Your AWS Account ID. | `secret env` | **Yes** | N/A |\n| `AWS_ACCESS_KEY_ID` | Your AWS Access Key. [More info here.](https://docs.aws.amazon.com/general/latest/gr/managing-aws-access-keys.html) | `secret env` | **Yes** | N/A |\n| `AWS_SECRET_ACCESS_KEY` | Your AWS Secret Access Key. [More info here.](https://docs.aws.amazon.com/general/latest/gr/managing-aws-access-keys.html) | `secret env` | **Yes** | N/A |\n| `AWS_REGION` | The region you want the VPC Stack to live in. | `env` | **Yes** | N/A |\n| `DEPLOY_ID` | SHA of the commit that triggered the action. | `env` / `github.sha` | **Yes** | N/A |\n| `ACRUL_CONFIG_PATH` | Path to the ACRU-L configuration file to use. | `env` | No | `./acru-l.toml` |\n\n\n## License\n\nThis project is distributed under the [MIT license](LICENSE).\n\n\n## Why?\nThe problem with infrastructure as code ...\n\nMonorepos...\nSnowflake code...\n\nConfounding application source code with devops\n\nA strict interface and reuse patterns\n\n## Installation\n\n`poetry add -D acru-l`\n\n`pip install acru-l`\n\n\n\n## About\n\n### Core Concepts\n\n* Resources - Extended constructs\n* Services - Collections of Resources that build a service interface\n* Stacks - Collections of Services\n\n#### Resources\nExtended constructs with set defaults\n\n#### Services\n\nTBD\n\n#### Stacks\n\nTBD\n',
    'author': 'Anthony Almarza',
    'author_email': 'anthony.almarza@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



# https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ses.html
import boto3
import environs

from acrul_toolkit.custom_resources import CustomResourceEventHandler


env = environs.Env()


class EventHandler(CustomResourceEventHandler):

    configuration_set_name = env("CONFIGURATION_SET_NAME")
    event_destination_name = env("EVENT_DESTINATION_NAME")

    def __init__(self):
        self.client = boto3.client("ses")

    def on_create(self, event):
        self.create_configuration_set(event)
        self.create_event_destination(event)

    def on_update(self, event):
        self.update_event_destination(event)

    def on_delete(self, event):
        self.delete_event_destination(event)
        self.delete_configuration_set(event)

    @staticmethod
    def get_topic_arn(event):
        return event["ResourceProperties"]["TopicARN"]

    def create_configuration_set(self, event):
        return self.client.create_configuration_set(
            ConfigurationSet={"Name": self.configuration_set_name}
        )

    def delete_configuration_set(self, event):
        return self.client.delete_configuration_set(
            ConfigurationSetName=self.configuration_set_name
        )

    def create_event_destination(self, event):
        return self.client.create_configuration_set_event_destination(
            **self.get_destination_config(event)
        )

    def update_event_destination(self, event):
        return self.client.update_configuration_set_event_destination(
            **self.get_destination_config(event)
        )

    def delete_event_destination(self, event):
        return self.client.delete_configuration_set_event_destination(
            ConfigurationSetName=self.configuration_set_name,
            EventDestinationName=self.event_destination_name,
        )

    def get_destination_config(self, event):
        return dict(
            ConfigurationSetName=self.configuration_set_name,
            EventDestination={
                "Name": self.event_destination_name,
                "Enabled": True,
                "MatchingEventTypes": [
                    "send",
                    "reject",
                    "bounce",
                    "complaint",
                    "delivery",
                    "open",
                    "click",
                    "renderingFailure",
                ],
                "SNSDestination": {"TopicARN": self.get_topic_arn(event)},
            },
        )


main = EventHandler()
